# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.
#
# Author: Benjamin Kampmann <benjamin@fluendo.com>

from elisa.core import common

from elisa.plugins.poblesec.base.list_switcher import ListSwitcherController

from elisa.plugins.poblesec.base.hierarchy import HierarchyController

from elisa.plugins.database.photo_controller import PhotoVerticalWithPreview, \
    PhotoCoverflow, PhotoGrid
from elisa.plugins.database.photo_controller import GenericPhotosController

from elisa.plugins.poblesec.base.preview_list import \
    MenuItemPreviewListController, DoubleLineMenuItemPreviewListController
from elisa.plugins.poblesec.base.coverflow import \
    ImageWithReflectionCoverflowController
from elisa.plugins.poblesec.base.grid import GridItemGridController

from elisa.plugins.database.music_controller import GenericTracksDbController, \
                                              DBTracksViewMode

from elisa.plugins.database.photo_controller import PhotosController, \
                                              PhotoViewMode
                                              
from elisa.plugins.database.video_controller import VideoController, \
                                              VideoViewMode
from elisa.plugins.poblesec.link import Link

from elisa.plugins.database.models import MusicTrack, File, Image, Video
from storm.expr import Desc

from twisted.internet import defer

# i18n
from elisa.core.utils.i18n import install_translation
_ = install_translation('database')


def most_played_tracks_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/music/tracks_played'
    link.label = _('Most Played Tracks')
    link.controller_args = {'limit': 20,}
    link.icon = 'elisa.plugins.poblesec.playlist'
    controller.model.append(link)

    return defer.succeed(None)


def last_played_tracks_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/music/tracks_played'
    link.label = _('Recently Played Tracks')
    link.controller_args = {'limit': 20, 'order_by': 'last_played'}
    link.icon = 'elisa.plugins.poblesec.playlist'
    controller.model.append(link)

    return defer.succeed(None)

def recently_added_video_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/video/video_added'
    link.label = _('Recently Added Videos')
    link.controller_args = {'limit': 20,}
    link.icon = 'elisa.plugins.poblesec.video_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def most_played_video_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/video/video_played'
    link.label = _('Most Watched Videos')
    link.controller_args = {'limit': 20,}
    link.icon = 'elisa.plugins.poblesec.video_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def last_played_video_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/video/video_played'
    link.label = _('Last Watched Videos')
    link.controller_args = {'limit': 20, 'order_by': 'last_played'}
    link.icon = 'elisa.plugins.poblesec.video_playlist'
    controller.model.append(link)

    return defer.succeed(None)
    
def unplayed_video_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/video/video_unplayed'
    link.label = _('Unwatched Videos')
    link.controller_args = {'limit': 20,}
    link.icon = 'elisa.plugins.poblesec.video_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def recently_added_tracks_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/music/tracks_added'
    link.label = _('Recently Added Tracks')
    link.controller_args = {'limit': 20,}
    link.icon = 'elisa.plugins.poblesec.playlist'
    controller.model.append(link)

    return defer.succeed(None)

def music_playlists_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/music/playlists'
    link.label = _('Playlists')
    link.icon = 'elisa.plugins.poblesec.playlist'
    controller.model.append(link)

    return defer.succeed(None)

def video_playlists_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/video/playlists'
    link.label = _('Playlists')
    link.icon = 'elisa.plugins.poblesec.video_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def recently_added_photos_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/photo/photos_added'
    link.label = _('Recently Added Photos')
    link.controller_args = {'limit': 20}
    link.icon = 'elisa.plugins.poblesec.photo_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def last_played_photos_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/photo/photos_played'
    link.label = _('Recently Viewed Photos')
    link.controller_args = {'limit': 20,  'order_by' : 'last_played'}
    link.icon = 'elisa.plugins.poblesec.photo_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def most_played_photos_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/photo/photos_played'
    link.label = _('Most Viewed Photos')
    link.controller_args = {'limit': 20,}
    link.icon = 'elisa.plugins.poblesec.photo_playlist'
    controller.model.append(link)

    return defer.succeed(None)

def pictures_playlists_decorator(controller):
    link = Link()
    link.controller_path = '/poblesec/database/pictures/playlists'
    link.label = _('History')
    link.icon = 'elisa.plugins.poblesec.photo_playlist'
    controller.model.append(link)

    return defer.succeed(None)


class PlayedMixin(object):
    def _load(self, result, filter, order_by, limit):
        self.filter = filter

        def got_items(models):
            self.model.extend(models)
            return self

        def get_items(result_set):
            return result_set.all()

        def set_limit(result_set, limit):
            result_set.config(limit=limit)
            return result_set

        def sort_by_count(result_set):
            result_set.order_by(Desc(File.playcount))
            return result_set

        def sort_by_date(result_set):
            result_set.order_by(Desc(File.last_played))
            return result_set

        store = common.application.store

        dfr = store.find(filter, filter.file_path == File.path,
                         File.playcount != 0, File.hidden == False)

        if order_by == "last_played":
            dfr.addCallback(sort_by_date)
        else:
            dfr.addCallback(sort_by_count)

        dfr.addCallback(set_limit, limit)
        dfr.addCallback(get_items)
        dfr.addCallback(got_items)
        return dfr

class AddedMixin(object):
    def _load(self, result, filter, limit):
        self.filter = filter

        def got_items(models):
            self.model.extend(models)
            return self

        def get_items(result_set):
            return result_set.all()

        def set_limit(result_set, limit):
            result_set.config(limit=limit)
            return result_set

        def sort_by_added(result_set):
            # this is not exactly the time it was added but last changed
            result_set.order_by(Desc(File.modification_time))
            return result_set

        store = common.application.store

        dfr = store.find(filter, filter.file_path == File.path,
                         File.hidden == False)

        dfr.addCallback(sort_by_added)

        dfr.addCallback(set_limit, limit)
        dfr.addCallback(get_items)
        dfr.addCallback(got_items)
        return dfr
        
class UnPlayedMixin(object):
    def _load(self, result, filter, limit):
        self.filter = filter

        def got_items(models):
            self.model.extend(models)
            return self

        def get_items(result_set):
            return result_set.all()

        def set_limit(result_set, limit):
            result_set.config(limit=limit)
            return result_set

        store = common.application.store

        dfr = store.find(filter, filter.file_path == File.path, \
                File.playcount == 0)

        dfr.addCallback(set_limit, limit)
        dfr.addCallback(get_items)
        dfr.addCallback(got_items)
        return dfr

class PlayedTracksController(GenericTracksDbController, PlayedMixin):
    def initialize(self, order_by=None, limit=10):
        dfr = super(PlayedTracksController, self).initialize()

        dfr.addCallback(self._load, MusicTrack, order_by, limit)
        return dfr

class AddedTracksController(GenericTracksDbController, AddedMixin):

    def initialize(self, limit=10):
        dfr = super(AddedTracksController, self).initialize()

        dfr.addCallback(self._load, MusicTrack, limit)
        return dfr

class PlayedPhotosController(GenericPhotosController, PlayedMixin):
    def initialize(self, order_by=None, limit=10):
        dfr = super(PlayedPhotosController, self).initialize()

        dfr.addCallback(self._load, Image, order_by, limit)
        return dfr

class AddedPhotosController(GenericPhotosController, AddedMixin):

    def initialize(self, limit=10):
        dfr = super(AddedPhotosController, self).initialize()

        dfr.addCallback(self._load, Image, limit)
        return dfr
    
class AddedVideoController(AddedMixin, VideoController):
    def initialize(self, limit=10):
        dfr = super(AddedVideoController, self).initialize()

        dfr.addCallback(self._load, Video,limit)
        return dfr

class PlayedVideoController(PlayedMixin, VideoController):
     def initialize(self, order_by=None, limit=10):
        dfr = super(PlayedVideoController, self).initialize()

        dfr.addCallback(self._load, Video, order_by, limit)
        return dfr
        
class UnPlayedVideoController(UnPlayedMixin, VideoController):
     def initialize(self, limit=10):
        dfr = super(UnPlayedVideoController, self).initialize()

        dfr.addCallback(self._load, Video, limit)
        return dfr

# Added Tracks 
class AddedTracksVerticalWithPreview(AddedTracksController, DoubleLineMenuItemPreviewListController):
    view_mode = DBTracksViewMode

class AddedTracksCoverflow(AddedTracksController, ImageWithReflectionCoverflowController):
    view_mode = DBTracksViewMode

class AddedTracksGrid(AddedTracksController, GridItemGridController):
    view_mode = DBTracksViewMode

class AddedTracksListSwitcherController(ListSwitcherController):
    modes = [AddedTracksVerticalWithPreview,
             AddedTracksCoverflow,
             AddedTracksGrid]
    default_mode = AddedTracksVerticalWithPreview

# Added Video 
class AddedVideoVerticalWithPreview(AddedVideoController, MenuItemPreviewListController):
    view_mode = VideoViewMode

class AddedVideoCoverflow(AddedVideoController, ImageWithReflectionCoverflowController):
    view_mode = VideoViewMode

class AddedVideoGrid(AddedVideoController, GridItemGridController):
    view_mode = VideoViewMode

class AddedVideoListSwitcherController(ListSwitcherController):
    modes = [AddedVideoVerticalWithPreview,
             AddedVideoCoverflow,
             AddedVideoGrid]
    default_mode = AddedVideoVerticalWithPreview
    
# Played Video
class PlayedVideoVerticalWithPreview(PlayedVideoController, MenuItemPreviewListController):
    view_mode = VideoViewMode

class PlayedVideoCoverflow(PlayedVideoController, ImageWithReflectionCoverflowController):
    view_mode = VideoViewMode

class PlayedVideoGrid(PlayedVideoController, GridItemGridController):
    view_mode = VideoViewMode

class PlayedVideoListSwitcherController(ListSwitcherController):
    modes = [PlayedVideoVerticalWithPreview,
             PlayedVideoCoverflow,
             PlayedVideoGrid]
    default_mode = PlayedVideoVerticalWithPreview
    
# UnPlayed Video
class UnPlayedVideoVerticalWithPreview(UnPlayedVideoController, MenuItemPreviewListController):
    view_mode = VideoViewMode

class UnPlayedVideoCoverflow(UnPlayedVideoController, ImageWithReflectionCoverflowController):
    view_mode = VideoViewMode

class UnPlayedVideoGrid(UnPlayedVideoController, GridItemGridController):
    view_mode = VideoViewMode

class UnPlayedVideoListSwitcherController(ListSwitcherController):
    modes = [UnPlayedVideoVerticalWithPreview,
             UnPlayedVideoCoverflow,
             UnPlayedVideoGrid]
    default_mode = UnPlayedVideoVerticalWithPreview

# Played Tracks
class PlayedTracksVerticalWithPreview(PlayedTracksController, DoubleLineMenuItemPreviewListController):
    view_mode = DBTracksViewMode

class PlayedTracksCoverflow(PlayedTracksController, ImageWithReflectionCoverflowController):
    view_mode = DBTracksViewMode

class PlayedTracksGrid(PlayedTracksController, GridItemGridController):
    view_mode = DBTracksViewMode

class PlayedTracksListSwitcherController(ListSwitcherController):
    modes = [PlayedTracksVerticalWithPreview,
             PlayedTracksCoverflow,
             PlayedTracksGrid]
    default_mode = PlayedTracksVerticalWithPreview

# Added Photos 
class AddedPhotosVerticalWithPreview(AddedPhotosController, MenuItemPreviewListController):
    view_mode = PhotoViewMode

class AddedPhotosCoverflow(AddedPhotosController, ImageWithReflectionCoverflowController):
    view_mode = PhotoViewMode

class AddedPhotosGrid(AddedPhotosController, GridItemGridController):
    view_mode = PhotoViewMode

class AddedPhotosListSwitcherController(ListSwitcherController):
    modes = [AddedPhotosVerticalWithPreview,
             AddedPhotosCoverflow,
             AddedPhotosGrid]
    default_mode = AddedPhotosGrid

# Played Photos
class PlayedPhotosVerticalWithPreview(PlayedPhotosController, MenuItemPreviewListController):
    view_mode = PhotoViewMode

class PlayedPhotosCoverflow(PlayedPhotosController, ImageWithReflectionCoverflowController):
    view_mode = PhotoViewMode

class PlayedPhotosGrid(PlayedPhotosController, GridItemGridController):
    view_mode = PhotoViewMode

class PlayedPhotosListSwitcherController(ListSwitcherController):
    modes = [PlayedPhotosVerticalWithPreview,
             PlayedPhotosCoverflow,
             PlayedPhotosGrid]
    default_mode = PlayedPhotosGrid
